/**
 * Copyright © 2018 Elastic Path Software Inc. All rights reserved.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this license. If not, see
 *
 *     https://www.gnu.org/licenses/
 *
 *
 */

import React from 'react';
import PropTypes from 'prop-types';
import intl from 'react-intl-universal';
import Config from 'config';
import { login } from '../../common/AuthService';
import { cortexFetch } from '../../common/Cortex';

class CartLineItemCommercial extends React.Component {
  static propTypes = {
    item: PropTypes.objectOf(PropTypes.any).isRequired,
    handleQuantityChange: PropTypes.func.isRequired,
  }

  constructor(props) {
    super(props);
    const { item } = this.props;
    this.state = {
      quantity: item.quantity,
    };
    this.handleQuantityChange = this.handleQuantityChange.bind(this);
    this.handleQuantityDecrement = this.handleQuantityDecrement.bind(this);
    this.handleQuantityIncrement = this.handleQuantityIncrement.bind(this);
    this.handleMoveToCartBtnClicked = this.handleMoveToCartBtnClicked.bind(this);
    this.handleRemoveBtnClicked = this.handleRemoveBtnClicked.bind(this);
  }

  handleQuantityChange(event) {
    event.preventDefault();
    const { item, handleQuantityChange } = this.props;
    const { quantity } = this.state;
    if (quantity === '') {
      this.setState({ quantity: 1 });
    }
    login().then(() => {
      cortexFetch(item.self.uri,
        {
          method: 'put',
          headers: {
            'Content-Type': 'application/json',
            Authorization: localStorage.getItem(`${Config.cortexApi.scope}_oAuthToken`),
          },
          body: JSON.stringify({
            quantity,
          }),
        })
        .then(() => {
          handleQuantityChange();
        })
        .catch((error) => {
          // eslint-disable-next-line no-console
          console.error(error.message);
        });
    });
  }

  handleQuantityDecrement() {
    const { quantity } = this.state;
    if (quantity > 1) {
      const newItemQuantity = parseInt(quantity, 10) - 1;
      this.setState({ quantity: newItemQuantity });
    }
  }

  handleQuantityIncrement() {
    const { quantity } = this.state;
    const newItemQuantity = parseInt(quantity, 10) + 1;
    this.setState({ quantity: newItemQuantity });
  }

  handleMoveToCartBtnClicked() {
    const { item } = this.props;
    login().then(() => {
      const moveToCartLink = item._movetocartform[0].links.find(link => link.rel === 'movetocartaction');
      cortexFetch(moveToCartLink.uri,
        {
          method: 'post',
          headers: {
            'Content-Type': 'application/json',
            Authorization: localStorage.getItem(`${Config.cortexApi.scope}_oAuthToken`),
          },
          body: JSON.stringify({
            quantity: 1,
          }),
        })
        .then((res) => {
          if (res.status === 200 || res.status === 201) {
            window.location = '/mybag';
          }
        })
        .catch((error) => {
          // eslint-disable-next-line no-console
          console.error(error.message);
        });
    });
  }

  handleRemoveBtnClicked() {
    const { item, handleQuantityChange } = this.props;
    login().then(() => {
      cortexFetch(item.self.uri,
        {
          method: 'delete',
          headers: {
            'Content-Type': 'application/json',
            Authorization: localStorage.getItem(`${Config.cortexApi.scope}_oAuthToken`),
          },
        })
        .then(() => {
          handleQuantityChange();
        })
        .catch((error) => {
          // eslint-disable-next-line no-console
          console.error(error.message);
        });
    });
  }

  renderUnitPrice() {
    const { item } = this.props;
    const itemPrice = ((item._price) ? (item._price) : (item._item[0]._price));
    const listPrice = itemPrice[0]['list-price'][0].display;
    const purchasePrice = itemPrice[0]['purchase-price'][0].display;
    if (listPrice !== purchasePrice) {
      return (
        <ul className="price-container">
          <li className="cart-unit-list-price" data-region="itemListPriceRegion">
            {listPrice}
          </li>
          <li className="cart-unit-purchase-price">
            {purchasePrice}
          </li>
        </ul>
      );
    }
    return (
      <ul className="price-container">
        <li className="cart-unit-purchase-price">
          {purchasePrice}
        </li>
      </ul>
    );
  }

  renderTotalPrice() {
    const { item } = this.props;
    const itemTotal = ((item._total) ? (item._total[0].cost[0].display) : (''));
    return (
      <ul className="price-container">
        <li className="cart-total-list-price is-hidden" data-region="itemListPriceRegion" />
        <li className="cart-total-purchase-price">
          {itemTotal}
        </li>
      </ul>
    );
  }

  renderConfiguration() {
    const { item } = this.props;
    const keys = Object.keys(item.configuration);
    if (keys) {
      return keys.map(key => (
        <li className="configuration" key={key}>
          <label htmlFor="option-name" className="option-name">
            {key}
            :&nbsp;
          </label>
          <span>
            {item.configuration[key]}
          </span>
        </li>
      ));
    }
    return null;
  }

  renderOptions() {
    const { item } = this.props;
    const options = item._item[0]._definition[0]._options;
    if (options) {
      return (
        options[0]._element.map(option => (
          <li className="option" key={option['display-name']}>
            <label htmlFor="option-value" className="option-name">
              {option['display-name']}
              :&nbsp;
            </label>
            <span className="option-value">
              {option._value[0]['display-name']}
            </span>
          </li>
        ))
      );
    }
    return null;
  }

  render() {
    const { item } = this.props;
    const { quantity } = this.state;
    const itemAvailability = ((item._availability) ? (item._availability) : (item._item[0]._availability));
    let availability = (itemAvailability[0].state === 'AVAILABLE');
    let availabilityString = '';
    if (itemAvailability.length >= 0) {
      if (itemAvailability[0].state === 'AVAILABLE') {
        availability = true;
        availabilityString = intl.get('in-stock');
      } else if (itemAvailability[0].state === 'AVAILABLE_FOR_PRE_ORDER') {
        availability = true;
        availabilityString = intl.get('pre-order');
      } else if (itemAvailability[0].state === 'AVAILABLE_FOR_BACK_ORDER') {
        availability = true;
        availabilityString = intl.get('back-order');
      } else {
        availability = false;
        availabilityString = intl.get('out-of-stock');
      }
    }
    return (
      <div className="cart-lineitem-row">
        <div className="row">
          <div className="cart-col-prd col-md">
            <div className="title-col" data-el-value="lineItem.displayName">
              {item._item[0]._definition[0]['display-name']}
            </div>
          </div>
          <div className="options-col col-md">
            <ul className="options-container">
              {this.renderOptions()}
              {this.renderConfiguration()}
            </ul>
          </div>
          <div className="cart-col-availability col-md">
            <div className="availability-col" data-region="cartLineitemAvailabilityRegion">
              <ul className="availability-container">
                <li className="availability itemdetail-availability-state" data-i18n="AVAILABLE">
                  <div>
                    {availability && <span className="icon" />}
                    {availabilityString}
                  </div>
                </li>
                <li className={`category-item-release-date${itemAvailability[0]['release-date'] ? '' : ' is-hidden'}`} data-region="itemAvailabilityDescriptionRegion">
                  <label htmlFor="release-date-value" className="releasedate-label">
                    {intl.get('expected-release-date')}
                    :&nbsp;
                  </label>
                  <span className="release-date-value">
                    {(itemAvailability[0]['release-date']) ? itemAvailability[0]['release-date']['display-value'] : ''}
                  </span>
                </li>
              </ul>
            </div>
          </div>
          <div className="cart-col-qty col-md">
            <form className="quantity-col form-content" onSubmit={this.handleQuantityChange}>
              {(quantity !== undefined) ? [
                <span className="input-group-btn">
                  <button type="button" className="quantity-right-plus btn btn-number" data-type="plus" data-field="" onClick={this.handleQuantityIncrement}>
                    <span className="glyphicon glyphicon-plus" />
                  </button>
                  <div className="quantity-col form-content form-content-quantity">
                    <input className="product-display-item-quantity-select form-control form-control-quantity" type="number" step="1" min="1" max="9999" value={quantity} onChange={e => this.setState({ quantity: e.target.value })} />
                  </div>
                  <button type="button" className="quantity-left-minus btn btn-number" data-type="minus" data-field="" onClick={this.handleQuantityDecrement}>
                    <span className="glyphicon glyphicon-minus" />
                  </button>
                </span>,
                <input key="product-display-item-quantity-update-button" className="product-display-item-quantity-update-button" type="submit" value="Update Quantity" />,
              ] : ('')
              }
            </form>
          </div>
          <div className="cart-col-each col-md">
            <div className="unit-price-col" data-region="cartLineitemUnitPriceRegion">
              <div>
                <div data-region="itemUnitPriceRegion" style={{ display: 'block' }}>
                  <span className="mobile-title text-uppercase">
                    {intl.get('unit-price')}
                  </span>
                  {this.renderUnitPrice()}
                </div>
              </div>
            </div>
          </div>
          <div className="cart-col-total col-md">
            <div className="total-price-col" data-region="cartLineitemTotalPriceRegion">
              <div>
                <div data-region="itemTotalPriceRegion" style={{ display: 'block' }}>
                  <span className="mobile-title text-uppercase">
                    {intl.get('total-price')}
                  </span>
                  {this.renderTotalPrice()}
                </div>
                <div data-region="itemTotalRateRegion" />
              </div>
            </div>
          </div>
        </div>
        <div className="remove-btn-col">
          <button className="ep-btn small btn-cart-removelineitem" type="button" onClick={this.handleRemoveBtnClicked}>
            <span className="btn-text">
              {intl.get('remove')}
            </span>
          </button>
        </div>
        {(item._movetocartform) ? (
          <div className="move-to-cart-btn-col">
            <button className="ep-btn primary small btn-cart-moveToCart" type="button" onClick={this.handleMoveToCartBtnClicked}>
              <span className="btn-text">
                {intl.get('move-to-cart')}
              </span>
            </button>
          </div>
        ) : ('')
        }
      </div>
    );
  }
}

export default CartLineItemCommercial;
